/* GNIO - GLib Network Layer of GIO
 *
 * Copyright (C) 2008 Christian Kellner, Samuel Cormier-Iijima
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Authors: Christian Kellner <gicmo@gnome.org>
 *          Samuel Cormier-Iijima <sciyoshi@gmail.com>
 */

#include <config.h>
#include <glib.h>

#include "ginetaddress.h"

/**
 * SECTION:ginetaddress
 * @short_description: Base class for implementing IPv4/IPv6 classes
 *
 * 
 * 
 **/

G_DEFINE_ABSTRACT_TYPE (GInetAddress, g_inet_address, G_TYPE_INITIALLY_UNOWNED);

enum
{
  PROP_0,
  PROP_IS_ANY,
  PROP_IS_LINKLOCAL,
  PROP_IS_LOOPBACK,
  PROP_IS_SITELOCAL,
  PROP_IS_MULTICAST,
  PROP_IS_MC_GLOBAL,
  PROP_IS_MC_LINKLOCAL,
  PROP_IS_MC_NODELOCAL,
  PROP_IS_MC_ORGLOCAL,
  PROP_IS_MC_SITELOCAL,
};

static void
g_inet_address_get_property (GObject    *object,
                             guint       prop_id,
                             GValue     *value,
                             GParamSpec *pspec)
{
  GInetAddress *address = G_INET_ADDRESS (object);

  switch (prop_id)
    {
      case PROP_IS_ANY:
        g_value_set_boolean (value, G_INET_ADDRESS_GET_CLASS (address)->is_any (address));
        break;

      case PROP_IS_LINKLOCAL:
        g_value_set_boolean (value, G_INET_ADDRESS_GET_CLASS (address)->is_linklocal (address));
        break;

      case PROP_IS_LOOPBACK:
        g_value_set_boolean (value, G_INET_ADDRESS_GET_CLASS (address)->is_loopback (address));
        break;

      default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
    }
}

static void
g_inet_address_set_property (GObject      *object,
                             guint         prop_id,
                             const GValue *value,
                             GParamSpec   *pspec)
{
  GInetAddress *address G_GNUC_UNUSED = G_INET_ADDRESS (object);

  switch (prop_id)
    {
      default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
    }
}

static void
g_inet_address_class_init (GInetAddressClass *klass)
{
  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

  gobject_class->get_property = g_inet_address_get_property;
  gobject_class->set_property = g_inet_address_set_property;

  g_object_class_install_property (gobject_class, PROP_IS_ANY,
                                   g_param_spec_boolean ("is-any",
                                                         "",
                                                         "",
                                                         FALSE,
                                                         G_PARAM_READABLE | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NAME));

  g_object_class_install_property (gobject_class, PROP_IS_LINKLOCAL,
                                   g_param_spec_boolean ("is-link-local",
                                                         "",
                                                         "",
                                                         FALSE,
                                                         G_PARAM_READABLE | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NAME));

  g_object_class_install_property (gobject_class, PROP_IS_LOOPBACK,
                                   g_param_spec_boolean ("is-loopback",
                                                         "",
                                                         "",
                                                         FALSE,
                                                         G_PARAM_READABLE | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NAME));

  g_object_class_install_property (gobject_class, PROP_IS_SITELOCAL,
                                   g_param_spec_boolean ("is-site-local",
                                                         "",
                                                         "",
                                                         FALSE,
                                                         G_PARAM_READABLE | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NAME));

  g_object_class_install_property (gobject_class, PROP_IS_MULTICAST,
                                   g_param_spec_boolean ("is-multicast",
                                                         "",
                                                         "",
                                                         FALSE,
                                                         G_PARAM_READABLE | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NAME));

  g_object_class_install_property (gobject_class, PROP_IS_MC_GLOBAL,
                                   g_param_spec_boolean ("is-mc-global",
                                                         "",
                                                         "",
                                                         FALSE,
                                                         G_PARAM_READABLE | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NAME));

  g_object_class_install_property (gobject_class, PROP_IS_MC_LINKLOCAL,
                                   g_param_spec_boolean ("is-mc-link-local",
                                                         "",
                                                         "",
                                                         FALSE,
                                                         G_PARAM_READABLE | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NAME));

  g_object_class_install_property (gobject_class, PROP_IS_MC_NODELOCAL,
                                   g_param_spec_boolean ("is-mc-node-local",
                                                         "",
                                                         "",
                                                         FALSE,
                                                         G_PARAM_READABLE | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NAME));

  g_object_class_install_property (gobject_class, PROP_IS_MC_ORGLOCAL,
                                   g_param_spec_boolean ("is-mc-org-local",
                                                         "",
                                                         "",
                                                         FALSE,
                                                         G_PARAM_READABLE | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NAME));

  g_object_class_install_property (gobject_class, PROP_IS_MC_SITELOCAL,
                                   g_param_spec_boolean ("is-mc-site-local",
                                                         "",
                                                         "",
                                                         FALSE,
                                                         G_PARAM_READABLE | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NAME));
}

static void
g_inet_address_init (GInetAddress *address)
{

}

gchar *
g_inet_address_to_string (GInetAddress *address)
{
  g_return_val_if_fail (G_IS_INET_ADDRESS (address), NULL);

  return G_INET_ADDRESS_GET_CLASS (address)->to_string (address);
}

/* ******************************************* */
/* Getters for properties */

static gboolean
get_boolean_property (GInetAddress *address, const gchar *property)
{
  gboolean value;

  g_return_val_if_fail (G_IS_INET_ADDRESS (address), FALSE);

  g_object_get (address, property, &value, NULL);

  return value;
}

gboolean
g_inet_address_is_any (GInetAddress *address)
{
  return get_boolean_property (address, "is-any");
}

gboolean
g_inet_address_is_linklocal (GInetAddress *address)
{
  return get_boolean_property (address, "is-link-local");
}

gboolean
g_inet_address_is_loopback (GInetAddress *address)
{
  return get_boolean_property (address, "is-loopback");
}

gboolean
g_inet_address_is_sitelocal (GInetAddress *address)
{
  return get_boolean_property (address, "is-site-local");
}

gboolean
g_inet_address_is_multicast (GInetAddress *address)
{
  return get_boolean_property (address, "is-multicast");
}

gboolean
g_inet_address_is_mc_global (GInetAddress *address)
{
  return get_boolean_property (address, "is-mc-global");
}

gboolean
g_inet_address_is_mc_linklocal (GInetAddress *address)
{
  return get_boolean_property (address, "is-mc-link-local");
}

gboolean
g_inet_address_is_mc_nodelocal (GInetAddress *address)
{
  return get_boolean_property (address, "is-mc-node-local");
}

gboolean
g_inet_address_is_mc_orglocal  (GInetAddress *address)
{
  return get_boolean_property (address, "is-mc-org-local");
}

gboolean
g_inet_address_is_mc_sitelocal (GInetAddress *address)
{
  return get_boolean_property (address, "is-mc-site-local");
}

